import com.sun.source.tree.UsesTree;
import org.junit.Rule;
import org.junit.jupiter.api.Test;

import javax.print.attribute.standard.Finishings;
import java.lang.invoke.VarHandle;
import java.util.ArrayList;
import java.util.Random;


// Test for judging whether a play is legal

class RuleControllerTest {
    private static CardParser parser = new CardParser();

    @Test
    /**
     * 0. A magic player owning all cards cannot play wildDraw4 anyway
     * Test lots of combinations of color (r,g,b,y) & symbol (skip, reverse)
     */
    void test_Player_AllCards_CannotWildDraw4() throws Exception {
        Player player = playerAllCards();
        System.out.println(player.getCards());
        RuleController ruler = new RuleController();

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));


        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));
    }


    @Test
    /**
     * 1. if current allowed color is red, player with no red cards can use wild Draw 4
     */
    void test_Player_Red_N_WildDraw4_Y_UseWildDraw4() throws Exception {
        Player player = player_Red_N_WildDraw4_Y();
        RuleController ruler = new RuleController();

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(ruler.isValidPlay(player, 105, false));  // playable!

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("none");
        assert(ruler.isValidPlay(player, 105, false)); // playable!

        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));


        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));
    }

    @Test
    /**
     * 2. test player with all colors cannot use wildDraw4 anyway
     */

    void test_Player_AllColors_CannotUseWildDraw4() throws Exception {
        Player player = player_AllColor_Y_WildDraw4_Y();
        RuleController ruler = new RuleController();

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));


        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("skip");
        ruler.setNextPlayerSkiplevel(3);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("reverse");
        ruler.setNextPlayerSkiplevel(0);
        assert(!ruler.isValidPlay(player, 105, false));

        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("none");
        assert(!ruler.isValidPlay(player, 105, false));
    }



    @Test
    /**
     * 3. test previous card: red 8
     */
    void testSituation_Red8() throws Exception {
        RuleController ruler = new RuleController();
        Player player = playerAllCards(); // a player who owns all cards

        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("none");
        ruler.setMatchableNumber("8");

        ArrayList<Integer> validByRuler = getAllLegalCardsByRuler(ruler, player);
        ArrayList<Integer> groundTruth = groundTruthGenerator(ruler, player,
                ruler.getMatchableColor(), ruler.getMatchableNumber(), ruler.getMatchableSymbol());

        System.out.println(validByRuler);
        System.out.println(groundTruth);
        assert(validByRuler.equals(groundTruth));
    }


    @Test
    /**
     * 4. test previous card: yellow 0
     */
    void testSituation_Yellow0() throws Exception {
        RuleController ruler = new RuleController();
        Player player = playerAllCards(); // a player who owns all cards
        ruler.setMatchableColor("yellow");
        ruler.setMatchableSymbol("none");
        ruler.setMatchableNumber("0");
        ArrayList<Integer> validByRuler = getAllLegalCardsByRuler(ruler, player);
        ArrayList<Integer> groundTruth = groundTruthGenerator(ruler, player,
                ruler.getMatchableColor(), ruler.getMatchableNumber(), ruler.getMatchableSymbol());

        System.out.println(validByRuler);
        System.out.println(groundTruth);
        assert(validByRuler.equals(groundTruth));
    }

    @Test
    /**
     * 5. test previous card: blue skip
     */
    void testSituation_blueSkip() throws Exception {
        RuleController ruler = new RuleController();
        Player player = playerAllCards(); // a player who owns all cards
        ruler.setMatchableColor("blue");
        ruler.setMatchableSymbol("skip");
        ruler.setMatchableNumber("none");
        ruler.setNextPlayerSkiplevel(3);
        ArrayList<Integer> validByRuler = getAllLegalCardsByRuler(ruler, player);
        ArrayList<Integer> groundTruth = groundTruthGenerator(ruler, player,
                ruler.getMatchableColor(), ruler.getMatchableNumber(), ruler.getMatchableSymbol());

        System.out.println(validByRuler);
        System.out.println(groundTruth);
        assert(validByRuler.equals(groundTruth));
    }

    @Test
    /**
     * 6. test previous card: green reverse
     */
    void testSituation_greenReverse() throws Exception {
        RuleController ruler = new RuleController();
        Player player = playerAllCards(); // a player who owns all cards
        ruler.setMatchableColor("green");
        ruler.setMatchableSymbol("reverse");
        ruler.setMatchableNumber("none");
        ruler.setNextPlayerSkiplevel(0);
        ArrayList<Integer> validByRuler = getAllLegalCardsByRuler(ruler, player);
        ArrayList<Integer> groundTruth = groundTruthGenerator(ruler, player,
                ruler.getMatchableColor(), ruler.getMatchableNumber(), ruler.getMatchableSymbol());

        System.out.println(validByRuler);
        System.out.println(groundTruth);
        assert(validByRuler.equals(groundTruth));
    }

    @Test
    /**
     * 7. test previous card: red draw2
     */
    void testSituation_RedDraw2() throws Exception {
        RuleController ruler = new RuleController();
        Player player = playerAllCards(); // a player who owns all cards
        ruler.setMatchableColor("red");
        ruler.setMatchableSymbol("draw2");
        ruler.setMatchableNumber("none");
        ruler.setNextPlayerSkiplevel(1);
        ArrayList<Integer> validByRuler = getAllLegalCardsByRuler(ruler, player);
        ArrayList<Integer> groundTruth = groundTruthGenerator(ruler, player,
                ruler.getMatchableColor(), ruler.getMatchableNumber(), ruler.getMatchableSymbol());

        System.out.println(validByRuler);
        System.out.println(groundTruth);
        assert(validByRuler.equals(groundTruth));
    }




    @Test
    /**
     * 8. test previous card: wild with red as declared color
     */
    void testSituation_wildDeclaredRed() throws Exception {
        RuleController ruler = new RuleController();
        Player player = playerAllCards(); // a player who owns all cards
        ruler.setMatchableColor("red");  // only color should be matchable in this case
        ruler.setMatchableSymbol("none");
        ruler.setMatchableNumber("none");
        ruler.setNextPlayerSkiplevel(0);
        ArrayList<Integer> validByRuler = getAllLegalCardsByRuler(ruler, player);
        ArrayList<Integer> groundTruth = groundTruthGenerator(ruler, player,
                ruler.getMatchableColor(), ruler.getMatchableNumber(), ruler.getMatchableSymbol());

        System.out.println(validByRuler);
        System.out.println(groundTruth);
        assert(validByRuler.equals(groundTruth));
    }


    @Test
    /**
     * 9. test previous card: wildDraw4 with red as declared color
     */
    void testSituation_wildDraw4DeclaredRed() throws Exception {
        RuleController ruler = new RuleController();
        Player player = playerAllCards(); // a player who owns all cards
        ruler.setMatchableColor("red");  // only color should be matchable in this case
        ruler.setMatchableSymbol("none");
        ruler.setMatchableNumber("none");
        ruler.setNextPlayerSkiplevel(2);
        ArrayList<Integer> validByRuler = getAllLegalCardsByRuler(ruler, player);
        ArrayList<Integer> groundTruth = groundTruthGenerator(ruler, player,
                ruler.getMatchableColor(), ruler.getMatchableNumber(), ruler.getMatchableSymbol());

        System.out.println(validByRuler);
        System.out.println(groundTruth);
        assert(validByRuler.equals(groundTruth));
    }











    /**
     *  extract all the cards that the ruler think as legal
     */
    private ArrayList<Integer> getAllLegalCardsByRuler(RuleController ruler, Player player) {
        ArrayList<Integer> validList = new ArrayList<>();
        for (int cardID : player.getCards()) {
            if (ruler.isValidPlay(player, cardID, false)) {
                validList.add(cardID);
            }
        }
        return validList;
    }

    /**
     * extract all the cards that are indeed legal (ground true generator)
     */
    private ArrayList<Integer> groundTruthGenerator(RuleController ruler,
                                                    Player player,
                                                    String colorTruth,
                                                    String numberTruth,
                                                    String symbolTruth) {
        ArrayList<Integer> validList = new ArrayList<>();
        for (int cardID : player.getCards()) {
            String cardDescription = parser.parseCardID(cardID);
            String[] result = parser.parseCardDescription(cardDescription);
            String cardCol = result[0];
            String cardContent = result[2];

            if (ruler.getNextPlayerSkiplevel() == 3) {
                continue; // skip card played. any card will be illegal

            } else if (ruler.getNextPlayerSkiplevel() == 2) {
                // only wildDraw4 is allowed
                if (cardContent.equals("wildDraw4") && ruler.isValidPlay(player, cardID, false)) {
                     // this call is valid because wildDraw4 behavior has passed the test.
                    validList.add(cardID);
                }

            } else if (ruler.getNextPlayerSkiplevel() == 1) {
                // both draw 2 and wild draw 4 are allowed
                if (cardContent.equals("wildDraw4") && ruler.isValidPlay(player, cardID, false)) {
                    validList.add(cardID);
                } else if (cardContent.equals("draw2")) {
                    validList.add(cardID);
                }

            } else {
                // cards with any one of color, number or symbol matched are playable
                if (cardContent.equals("wild")) {
                    validList.add(cardID);
                } else if (cardContent.equals("wildDraw4") && ruler.isValidPlay(player, cardID, false)) {
                    validList.add(cardID);
                } else {
                    if (cardCol.equals(colorTruth)) {
                        validList.add(cardID);
                    } else if (cardContent.equals(numberTruth)) {
                        validList.add(cardID);
                    } else if (cardContent.equals(symbolTruth)) {
                        validList.add(cardID);
                    }
                }
            }
        }

        return validList;
    }

    /**
     * construct a player who has no red cards, but a wildDraw4 card
     * This player should be able to play wildDraw4 card if current allowed color is red
     * @return
     */
    private Player player_Red_N_WildDraw4_Y() {
        Player player = new Player(0, null);
        player.addOneCard(26); // green 1
        player.addOneCard(51); // blue 1
        player.addOneCard(76); // yellow 1
        player.addOneCard(101); // wild
        player.addOneCard(105); // wildDraw 4
        return player;
    }

    /**
     * Construct a player who owns all colors, and also a wildDraw4 card
     * This player should not be able to play wildDraw4 card.
     * @return
     */
    private Player player_AllColor_Y_WildDraw4_Y() {
        Player player = new Player(0, null);
        player.addOneCard(1); // red 1
        player.addOneCard(26); // green 1
        player.addOneCard(51); // blue 1
        player.addOneCard(76); // yellow 1
        player.addOneCard(101); // wild
        player.addOneCard(105); // wildDraw 4
        return player;
    }

    /**
     * Construct a magic player who own ALL CARDS
     * This player should not be able to play wildDraw4 card
     */
    private Player playerAllCards() {
        Player player = new Player(0, null);
        for (int i = 1; i <= 108; i++) {
            player.addOneCard(i);
        }
        return player;
    }
}