import java.lang.reflect.Array;
import java.util.*;

public class CardManager {
    private ArrayList<Integer> cardPile;
    private ArrayList<Integer> discardPile;
    private static CardParser parser;  // only for debug purpose...

    public CardManager() {
        parser = new CardParser();
        discardPile = new ArrayList<>();
        initializeCardPile();
        printCardPile();
    }

    private void initializeCardPile() {
        cardPile = new ArrayList<Integer>();
        for (int i = 1; i <= 108; i++) {
            cardPile.add(i);
        }
        Collections.shuffle(cardPile);
    }


    public void printCardPile() {
        if (!cardPile.isEmpty()) {
            System.out.println("\n\n===================================================================");
            System.out.println("Current Card Pile has " + cardPile.size() + " card left:");
            for (int i = 0; i < cardPile.size(); i++) {
                System.out.println(parser.parseCardID(cardPile.get(i)));
            }
            System.out.println("===================================================================\n\n");
        } else {
            System.out.println("============================================================");
            System.out.println("There are no card left in the pile!");
        }
    }

    public int numCardLeft() {
        return cardPile.size();
    }

    /**
     *  draw a certain number of cards from the card pile
     * @param numToDraw total number to draw. Should be positive integer
     * @return an arraylist of card drawn from the pile
     */
    public ArrayList<Integer> drawCards(int numToDraw) {
        if (numToDraw < 0) {
            System.out.println("drawCard error: Input number must be a positive integer");
            return null;
        }

        ArrayList<Integer> cardsDrawnFromDiscard = new ArrayList<Integer>();
        if (numCardLeft() < numToDraw) {
            int numToDrawFromDiscard = numToDraw - cardPile.size();
            cardsDrawnFromDiscard = drawCardsfromDiscardPile(numToDrawFromDiscard);
        }

        ArrayList<Integer> drawnCards = new ArrayList<Integer>(cardPile.subList(0, numToDraw));
        drawnCards.addAll(cardsDrawnFromDiscard);
        cardPile.subList(0, numToDraw).clear(); // erase the drawn cards
        return drawnCards;
    }

    private ArrayList<Integer> drawCardsfromDiscardPile(int numToDraw) {
        ArrayList<Integer> drawnCards = new ArrayList<Integer>(discardPile.subList(0, numToDraw));
        cardPile.subList(0, numToDraw).clear(); // erase the drawn cards
        return drawnCards;

    }

    /**
     * @parm  ID of the card to be inserted
     */
    public void insertOneCardToDiscardPile(int cardID) {
        discardPile.add(0, cardID);
    }


    /**
     * Return number of card left on discard pile
     */
    public int numLeftDiscardPile() { return discardPile.size(); }

    /**
     * get all the cards currently in card pile as a Array list
     */
    public ArrayList<Integer> getCardPile() { return cardPile; }

}

