import java.util.*;

class CardParser {
    public final int NUMBER_PER_COLOR = 25;
    public final int TOTAL_COLORS = 4;
    public final int COLORED_CARD_NUM = NUMBER_PER_COLOR * TOTAL_COLORS;
    public HashMap<Integer, String> colorDict;

    public CardParser() {
        colorDict = new HashMap<Integer, String>();
        colorDict.put(0, "none");
        colorDict.put(1, "red");
        colorDict.put(2, "green");
        colorDict.put(3, "blue");
        colorDict.put(4, "yellow");
    }


    /**
     * Given ID of the card, return its content as a string as "color function" - e.g. "green 1", "none wild"
     * @param cardID a integer from 1 - 108, unique id for each card
     * @return
     */
    public String parseCardID(int cardID) {
        if (cardID < 0) {
            // invald cardID
            return "NA NA NA";
        }

        if (cardID > COLORED_CARD_NUM) {
            return parseWildCard(cardID);
        } else {
            return parseColoredCard(cardID);
        }
    }

    /**
     * helper function for parse for cards without colors
     */
    private String parseWildCard(int cardID) {
        if (cardID > COLORED_CARD_NUM && cardID <= COLORED_CARD_NUM + 4) {
            return "NA sym wild";

        }  else if (cardID > COLORED_CARD_NUM + 4 && cardID <= COLORED_CARD_NUM + 8) {
            return "NA sym wildDraw4";
        }
        return "NA NA NA";
    }

    /**
     * helper function for parse for cards with colors
     */
    private String parseColoredCard(int cardID) {
        int colorID = (cardID - 1) / NUMBER_PER_COLOR + 1;
        String color = colorDict.get(colorID);
        Integer cardTypeID = cardID % NUMBER_PER_COLOR;

        String type;
        String content = "";
        if (cardTypeID <= 18) {
            // case 1 - 9
            type = "num";
            cardTypeID = cardTypeID <= 9 ? cardTypeID : (cardTypeID - 9);
            content = cardTypeID.toString();

        } else if (cardTypeID > 18 && cardTypeID <= 20) {
            // case skip card
            type = "sym";
            content = "skip";

        } else if (cardTypeID > 20 && cardTypeID <= 22) {
            // case reverse card
            type = "sym";
            content = "reverse";

        } else if (cardTypeID > 22 && cardTypeID <= 24 ) {
            // case draw 2
            type = "sym";
            content = "draw2";

        } else {
            // case 0
            type = "num";
            content = "0";
        }

        return constructCardDescription(color, type, content);
    }


    private String constructCardDescription(String color, String type, String value) {
        return color + " " + type + " " + value;
    }

    /**
     * Given the description of one card, parse its color type and content
     * @return a string array (String[]) : {color, type, content}
     */
    public String[] parseCardDescription(String description) {
        String[] results = description.split(" ", 3);
        String color = results[0];
        String type = results[1];
        String content = results[2];

        String[] output = {color, type, content};
        return output;
    }

    /**
     * judge whether a card is a wild card - that is, whether user can decalre a color
     * @param cardID id of card
     */
    public boolean isWildCard(int cardID) {
        return cardID > COLORED_CARD_NUM;
    }
}