import java.util.*;


/**
 * The class for recording Game state
 * Organize the interactions among all other classes
 */

public class Game {
    private static CardParser parser = new CardParser();
    private static final int INIT_DRAW = 7; // every player get 7 cards at beginning

    private int rounds = 1;
    public RuleController ruler;
    public CardManager gameCardManager;
    public ArrayList<Player> players;
    private int currentPlayerID; // playerID starts from 0 !!!!!

    /**
     * Default constructor for Game object
     * @param playerNum
     */
    public Game(int playerNum) {
        gameCardManager = new CardManager();
        ruler = new RuleController();
        decideFirstPlayerID(playerNum);

        System.out.println("Loading player information...");
        players = new ArrayList<Player>();

        // playerID starts from 0 !!!!!
        for (int i = 0; i < playerNum; i++) {
            Player player = new Player(i, this);
            player.drawCards(INIT_DRAW);
            players.add(player);
        }
    }


    /**
     * Randomly decide the start position of this game
     * @param playerNum total number of players
     */
    private void decideFirstPlayerID(int playerNum) {
        Random rand = new Random();
        currentPlayerID = rand.nextInt(playerNum);
    }


    /**
     * find the next player in the sequence, regardless whether he or not should be skipped
     */
    private void updateNextPlayerID() {
        if (ruler.getIsClockwise()) {
            currentPlayerID = (currentPlayerID + 1) % players.size();
        } else {
            currentPlayerID = (currentPlayerID - 1 + players.size()) % players.size();
        }

    }

    /**
     *  Run one round of UNO
     *  very import function !
     */
    private void runOneRound() {
        ruler.reportCurrentState(this);
        System.out.println("It's now Player " + (currentPlayerID + 1) + "'s turn...");
        Player currentPlayer = players.get(currentPlayerID);
        currentPlayer.playOneRound();

    }

    public void gameStart() {
        while (rounds <= 2) {
            System.out.println("\n\n\n\n\n\n\n\n\n\n\n\n\n<<<<<<<<<<<<<<<<<<<<<<<< Round" + rounds + " >>>>>>>>>>>>>>>>>>>>>>>>>\n\n");
            runOneRound();
            updateNextPlayerID();
            rounds++;
        }
    }
}
