import java.lang.reflect.Array;
import java.util.*;

public class Player {
    public static CardParser parser;
    private final ArrayList<Integer> cards;
    public int playerID;
    public static Game gameController;
    public static RuleController ruler;
    public CmdUI prompterCmd;
    public GUI prompterGUI;


    public Player(int ID, Game game) {
        playerID = ID;
        cards = new ArrayList<Integer>();
        gameController = game;
        ruler = (game != null) ? game.ruler : null;  // for testing purpose
        parser = (game != null) ? RuleController.parser : null;
        prompterCmd = new CmdUI(this);
        prompterGUI = null;
    }

    /**
     * @param numToDraw number of cards to draw
     */
    public void drawCards(int numToDraw) {
        ArrayList<Integer> newCards = gameController.gameCardManager.drawCards(numToDraw); // Initial draw
        cards.addAll(newCards);
    }

    /**
     * print all cards that this user has
     */
    public void printCardsInHand() {
        if (!cards.isEmpty()) {
            System.out.println("\n\n===================================================================");
            System.out.println("Currently have " + cards.size() + " cards:");
            for (int i = 0; i < cards.size(); i++) {
                System.out.println("[" + (i + 1) +"]   " + parser.parseCardID(cards.get(i)));
            }
            System.out.println("===================================================================\n\n");

        } else {
            System.out.println("============================================================");
            System.out.println("You got not cards in hands! Congrats, you are the winner!");
        }
    }

    /**
     *  iterate through cards and see if any of the player's cards are playable
     * @return an array list of legal cards
     */
    public ArrayList<Integer> findLegalCard() {
        ArrayList<Integer> legalCards = new ArrayList<>();
        for (int cardID : cards) {
            if (ruler.isValidPlay(this, cardID, false)) legalCards.add(cardID);
        }
        return legalCards;
    }

    /**
     *  iterate through cards and see if any of the player's cards are playable
     * @return an array list of the **indices** of legal cards from all cards
     */
    public ArrayList<Integer> findLegalCardIndex() {
        ArrayList<Integer> legalCardIndices = new ArrayList<>();
        for (int i = 0; i < cards.size(); i++) {
            if (ruler.isValidPlay(this, cards.get(i), false)) legalCardIndices.add(i);
        }
        return legalCardIndices;
    }


    /**
     * print all cards that this user has
     */
    public void printLegalCards() {
        ArrayList<Integer> legals = findLegalCard();
        if (!legals.isEmpty()) {
            System.out.println("\n\n===================================================================");
            System.out.println("The following cards are playable in current turn:");
            for (int i = 0; i < legals.size(); i++) {
                System.out.println("[" + (i + 1) +"]   " + parser.parseCardID(legals.get(i)));
            }
            System.out.println("===================================================================\n\n");

        } else {
            System.out.println("\n\n============================================================");
            System.out.println("You don't have any playable cards now.");
            System.out.println("============================================================\n\n");
        }
    }



    /**
     * getter for cards (private attribute for tracking all cards owned by a player)
     * @return the cards of the player
     */
    public ArrayList<Integer> getCards() {
        return cards;
    }


    /**
     * Caller for player to play one round
     * skip, draw, play ... all behavior will be handled by this function
     */
    public void playOneRound() {
        if (prompterGUI != null) {
            playOneRoundGUI();
        } else if (prompterCmd != null) {
            playOneRoundCmd();
        }
    }

    /**
     *  Playing with cmd as UI
     */
    public void playOneRoundCmd() {
        if (ruler.checkSkipandDraw(this)) {
            prompterCmd.printForcedSkip();
        } else {
            prompterCmd.promptTakeAction();
        }

    }

    /**
     *  TO BE IMPLEMENTED...
     */
    public void playOneRoundGUI() {
//        if (ruler.shoudPlayerBeSkipped(this)) {
//            prompterGUI
//        } else {}

    }

    /**
     * For Test only. Add one card with Given card ID to the player's cards
     */
    public void addOneCard(int cardID) {
        cards.add(cardID);
    }

}